## fcontextPage.py - show selinux mappings
## Copyright (C) 2006 Red Hat, Inc.

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

## Author: Dan Walsh
import gtk
import gtk.glade
import os
import libxml2
import gobject
import seobject
from semanagePage import *;
from avc import context

##
## I18N
## 
PROGNAME="system-config-selinux"

import gettext
gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)
try:
    gettext.install(PROGNAME, localedir="/usr/share/locale", unicode=1)
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode

class fcontextPage(semanagePage):
    def __init__(self, xml):
        semanagePage.__init__(self, xml, "fcontext", _("File Labeling"))
        self.view = xml.get_widget("fcontextView")
        self.store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.view.set_model(self.store)
#        self.store.set_sort_column_id(0, gtk.SORT_ASCENDING)        

        col = gtk.TreeViewColumn(_("File\nSpecification"), gtk.CellRendererText(), text=0)
	col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
	col.set_fixed_width(250)
        
        col.set_sort_column_id(0)
        col.set_resizable(True)
        self.view.append_column(col)
        col = gtk.TreeViewColumn(_("Selinux\nFile Context"), gtk.CellRendererText(), text=1)

	col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
	col.set_fixed_width(250)
        col.set_sort_column_id(1)
        col.set_resizable(True)
        self.view.append_column(col)
        col = gtk.TreeViewColumn(_("File\nType"), gtk.CellRendererText(), text=2)
        col.set_sort_column_id(2)
        col.set_resizable(True)
        self.view.append_column(col)
        self.load()
        self.fcontextEntry = xml.get_widget("fcontextEntry")
        self.fcontextFileTypeCombo = xml.get_widget("fcontextFileTypeCombo")
        liststore=self.fcontextFileTypeCombo.get_model()
        for k in seobject.file_types:
            if len(k) > 0 and  k[0] != '-':
                iter=liststore.append()
                liststore.set_value(iter, 0, k)
        iter = liststore.get_iter_first()
        self.fcontextFileTypeCombo.set_active_iter(iter)
        self.fcontextTypeEntry = xml.get_widget("fcontextTypeEntry")
        self.fcontextMLSEntry = xml.get_widget("fcontextMLSEntry")

    def load(self):
        self.fcontext=seobject.fcontextRecords()
        fcon_list=self.fcontext.get_all()
        self.store.clear()
        for fcon in fcon_list:
            iter=self.store.append()
            self.store.set_value(iter, 0, fcon[0])
            self.store.set_value(iter, 2, fcon[1])
            if len(fcon) > 3:
                rec="%s:%s:%s:%s " % (fcon[2], fcon[3],fcon[4], seobject.translate(fcon[5],False))
            else:
                rec="<<None>>"
            self.store.set_value(iter, 1, rec)
        self.view.get_selection().select_path ((0,))
    
    def dialogInit(self):
        store, iter = self.view.get_selection().get_selected()
        self.fcontextEntry.set_text(store.get_value(iter, 0))
        self.fcontextEntry.set_sensitive(False)
        scontext = store.get_value(iter, 1)
        scon=context(scontext)
        self.fcontextTypeEntry.set_text(scon.type)
        self.fcontextMLSEntry.set_text(scon.mls)
        type=store.get_value(iter, 2)
        liststore=self.fcontextFileTypeCombo.get_model()
        iter = liststore.get_iter_first()
        while iter != None and liststore.get_value(iter,0) != type:
            iter = liststore.iter_next(iter)
        if iter != None:
            self.fcontextFileTypeCombo.set_active_iter(iter)
        self.fcontextFileTypeCombo.set_sensitive(False)
        
    def dialogClear(self):
        self.fcontextEntry.set_text("")
        self.fcontextEntry.set_sensitive(True)
        self.fcontextFileTypeCombo.set_sensitive(True)
        self.fcontextTypeEntry.set_text("")
        self.fcontextMLSEntry.set_text("s0")
        
    def delete(self):
        store, iter = self.view.get_selection().get_selected()
        try:
            fspec=store.get_value(iter, 0)
            type=store.get_value(iter, 1)
            self.fcontext.delete(fspec, type)
            store.remove(iter)
            self.view.get_selection().select_path ((0,))
        except ValueError, e:
            self.error(e.args[0])

    def add(self):
        fspec=self.fcontextEntry.get_text().strip()
        type=self.fcontextTypeEntry.get_text().strip()
        mls=self.fcontextMLSEntry.get_text().strip()
        list_model=self.fcontextFileTypeCombo.get_model()
        iter = self.fcontextFileTypeCombo.get_active_iter()
        ftype=list_model.get_value(iter,0)

        self.fcontext.add(fspec, type, ftype, mls)

        iter=self.store.append()
        self.store.set_value(iter, 0, fspec)
        self.store.set_value(iter, 2, ftype)
        self.store.set_value(iter, 1, "system_u:object_r:%s:%s" % (type, mls))
        
    def modify(self):
        fspec=self.fcontextEntry.get_text().strip()
        type=self.fcontextTypeEntry.get_text().strip()
        mls=self.fcontextMLSEntry.get_text().strip()
        list_model=self.fcontextFileTypeCombo.get_model()
        iter = self.fcontextFileTypeCombo.get_active_iter()
        ftype=list_model.get_value(iter,0)
        self.fcontext.modify(fspec, type, ftype, mls, "")

        store, iter = self.view.get_selection().get_selected()
        self.store.set_value(iter, 0, fspec)
        self.store.set_value(iter, 2, ftype)
        self.store.set_value(iter, 1, "system_u:object_r:%s:%s" % (type, mls))
